@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -12,6 +12,7 @@ import inspect
 import unittest
 
 from traits.observation import expression
+from traits.observation._anytrait_filter import anytrait_filter
 from traits.observation._dict_item_observer import DictItemObserver
 from traits.observation._filtered_trait_observer import FilteredTraitObserver
 from traits.observation._list_item_observer import ListItemObserver
@@ -183,6 +184,52 @@ class TestObserverExpressionComposition(unittest.TestCase):
         self.assertEqual(actual, expected)
 
 
+class TestObserverExpressionAnytrait(unittest.TestCase):
+    """ Test anytrait function and method. """
+
+    def test_anytrait_function_notify_true(self):
+        expr = expression.anytrait(notify=True)
+        expected = [
+            create_graph(
+                FilteredTraitObserver(filter=anytrait_filter, notify=True),
+            ),
+        ]
+        actual = expr._as_graphs()
+        self.assertEqual(actual, expected)
+
+    def test_anytrait_function_notify_false(self):
+        expr = expression.anytrait(notify=False)
+        expected = [
+            create_graph(
+                FilteredTraitObserver(filter=anytrait_filter, notify=False),
+            ),
+        ]
+        actual = expr._as_graphs()
+        self.assertEqual(actual, expected)
+
+    def test_anytrait_method_notify_true(self):
+        expr = expression.trait("name").anytrait(notify=True)
+        expected = [
+            create_graph(
+                NamedTraitObserver(name="name", notify=True, optional=False),
+                FilteredTraitObserver(filter=anytrait_filter, notify=True),
+            ),
+        ]
+        actual = expr._as_graphs()
+        self.assertEqual(actual, expected)
+
+    def test_anytrait_method_notify_false(self):
+        expr = expression.trait("name").anytrait(notify=False)
+        expected = [
+            create_graph(
+                NamedTraitObserver(name="name", notify=True, optional=False),
+                FilteredTraitObserver(filter=anytrait_filter, notify=False),
+            ),
+        ]
+        actual = expr._as_graphs()
+        self.assertEqual(actual, expected)
+
+
 class TestObserverExpressionFilter(unittest.TestCase):
     """ Test ObserverExpression.match """
 
@@ -613,13 +660,14 @@ class TestObserverExpressionSetItem(unittest.TestCase):
         )
 
 
-class TestObserverExpressionEquality(unittest.TestCase):
-    """ Test ObserverExpression.__eq__ """
+class TestObserverExpressionEqualityAndHashing(unittest.TestCase):
+    """ Test ObserverExpression.__eq__ and ObserverExpression.__hash__. """
 
     def test_trait_equality(self):
         expr1 = create_expression(1)
         expr2 = create_expression(1)
         self.assertEqual(expr1, expr2)
+        self.assertEqual(hash(expr1), hash(expr2))
 
     def test_join_equality_with_then(self):
         # The following all result in the same graphs
@@ -630,7 +678,68 @@ class TestObserverExpressionEquality(unittest.TestCase):
         combined2 = expr1.then(expr2)
 
         self.assertEqual(combined1, combined2)
+        self.assertEqual(hash(combined1), hash(combined2))
+
+    def test_equality_of_parallel_expressions(self):
+        expr1 = create_expression(1) | create_expression(2)
+        expr2 = create_expression(1) | create_expression(2)
+        self.assertEqual(expr1, expr2)
+        self.assertEqual(hash(expr1), hash(expr2))
 
     def test_equality_different_type(self):
         expr = create_expression(1)
         self.assertNotEqual(expr, "1")
+
+
+class TestObserverExpressionSlots(unittest.TestCase):
+    """ Check that expressions use __slots__. """
+
+    def test_single_expression(self):
+        expr = create_expression(1)
+        self.assertFalse(hasattr(expr, "__dict__"))
+
+    def test_series_expression(self):
+        expr = create_expression(1).then(create_expression(2))
+        self.assertFalse(hasattr(expr, "__dict__"))
+
+    def test_parallel_expression(self):
+        expr = create_expression(1) | create_expression(2)
+        self.assertFalse(hasattr(expr, "__dict__"))
+
+
+class TestCompileFromExpr(unittest.TestCase):
+    """ Tests for compile_expr. """
+
+    # The complicated pieces are already tested; we just need to double
+    # check that "_as_graphs" corresponds to "compile_expr" for a
+    # handful of cases.
+
+    def test_compile_expr(self):
+        observer1 = 1
+        observer2 = 2
+        observer3 = 3
+        observer4 = 4
+        expr1 = create_expression(observer1)
+        expr2 = create_expression(observer2)
+        expr3 = create_expression(observer3)
+        expr4 = create_expression(observer4)
+
+        test_expressions = [
+            expr1,
+            expr1 | expr2,
+            expr1 | expr2 | expr3,
+            expr1.then(expr2),
+            expr1.then(expr2).then(expr3),
+            (expr1.then(expr2)) | (expr3.then(expr4)),
+            expr1.list_items(),
+            expr1.dict_items(),
+            expr1.set_items(),
+            expr1.anytrait(notify=False),
+            expr1.anytrait(notify=True),
+        ]
+        for test_expression in test_expressions:
+            with self.subTest(expression=test_expression):
+                self.assertEqual(
+                    expression.compile_expr(test_expression),
+                    test_expression._as_graphs(),
+                )
