@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -17,9 +17,11 @@ from traits.observation.api import (
     pop_exception_handler,
     push_exception_handler,
 )
-from traits.observation._exceptions import NotifierNotFound
-from traits.observation.expression import trait
+from traits.observation.exceptions import NotifierNotFound
+from traits.observation.expression import compile_expr, trait
 from traits.observation.observe import (
+    apply_observers,
+    dispatch_same,
     observe,
 )
 from traits.observation._observer_graph import ObserverGraph
@@ -556,3 +558,98 @@ class TestObserverIntegration(unittest.TestCase):
                 instance = ClassWithNumber()
                 instance.observe(callable_, "number")
                 instance.number += 1
+
+
+class TestApplyObservers(unittest.TestCase):
+    """ Test the public-facing apply_observers function."""
+
+    def setUp(self):
+        push_exception_handler(reraise_exceptions=True)
+        self.addCleanup(pop_exception_handler)
+
+    def test_apply_observers_with_expression(self):
+        foo = ClassWithNumber()
+        handler = mock.Mock()
+        graphs = compile_expr(trait("number"))
+
+        apply_observers(
+            object=foo,
+            graphs=graphs,
+            handler=handler,
+            dispatcher=dispatch_same,
+        )
+
+        # when
+        foo.number += 1
+
+        # then
+        self.assertEqual(handler.call_count, 1)
+        handler.reset_mock()
+
+        # when
+        apply_observers(
+            object=foo,
+            graphs=graphs,
+            handler=handler,
+            dispatcher=dispatch_same,
+            remove=True,
+        )
+        foo.number += 1
+
+        # then
+        self.assertEqual(handler.call_count, 0)
+
+    def test_apply_observers_different_dispatcher(self):
+
+        self.dispatch_records = []
+
+        def dispatcher(handler, event):
+            self.dispatch_records.append((handler, event))
+
+        foo = ClassWithNumber()
+        handler = mock.Mock()
+
+        # when
+        apply_observers(
+            object=foo,
+            graphs=compile_expr(trait("number")),
+            handler=handler,
+            dispatcher=dispatcher,
+        )
+        foo.number += 1
+
+        # then
+        # the dispatcher is called.
+        self.assertEqual(len(self.dispatch_records), 1)
+
+    def test_apply_observers_different_target(self):
+        # Test the result of setting target to be the same as object
+        parent1 = ClassWithInstance()
+        parent2 = ClassWithInstance()
+        graphs = compile_expr(trait("instance").trait("number"))
+
+        # the instance is shared
+        instance = ClassWithNumber()
+        parent1.instance = instance
+        parent2.instance = instance
+
+        handler = mock.Mock()
+
+        # when
+        apply_observers(
+            object=parent1,
+            graphs=graphs,
+            handler=handler,
+            dispatcher=dispatch_same,
+        )
+        apply_observers(
+            object=parent2,
+            graphs=graphs,
+            handler=handler,
+            dispatcher=dispatch_same,
+        )
+        instance.number += 1
+
+        # then
+        # the handler should be called twice as the targets are different.
+        self.assertEqual(handler.call_count, 2)
