@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -11,8 +11,11 @@
 """ Unit test case for testing interfaces and adaptation.
 """
 
+import contextlib
+import logging
 import unittest
 
+from traits import has_traits
 from traits.api import (
     HasTraits,
     Adapter,
@@ -27,6 +30,8 @@ from traits.api import (
     TraitError,
 )
 from traits.adaptation.api import reset_global_adaptation_manager
+from traits.constants import DefaultValue
+from traits.interface_checker import InterfaceError
 
 
 class IFoo(Interface):
@@ -114,6 +119,11 @@ class TraitsHolder(HasTraits):
     foo_plus_adapts_to = AdaptsTo(IFooPlus)
 
 
+class TraitsHolderSub(TraitsHolder):
+    foo_adapted_to = Sample()
+    foo_adapts_to = Sample()
+
+
 class SampleListAdapter(Adapter):
     def get_list(self):
         obj = self.adaptee
@@ -178,12 +188,32 @@ class InterfacesTest(unittest.TestCase):
     def test_provides_one(self):
         @provides(IFoo)
         class Test(HasTraits):
-            pass
+            def get_foo(self):
+                return "foo_and_only_foo"
+
+            def get_average(self):
+                return 42
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+        self.assertNotIsInstance(test, IAverage)
 
     def test_provides_multi(self):
         @provides(IFoo, IAverage, IList)
         class Test(HasTraits):
-            pass
+            def get_foo(self):
+                return "test_foo"
+
+            def get_average(self):
+                return 42
+
+            def get_list(self):
+                return [42]
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+        self.assertIsInstance(test, IAverage)
+        self.assertIsInstance(test, IList)
 
     def test_provides_extended(self):
         """ Ensure that subclasses of Interfaces imply the superinterface.
@@ -191,10 +221,19 @@ class InterfacesTest(unittest.TestCase):
 
         @provides(IFooPlus)
         class Test(HasTraits):
-            pass
+            def get_foo(self):
+                return "some_test_foo"
+
+            def get_foo_plus(self):
+                return "more_test_foo"
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+        self.assertIsInstance(test, IFooPlus)
 
         ta = TraitsHolder()
-        ta.foo_adapted_to = Test()
+        ta.foo_adapted_to = test
+        self.assertIs(ta.foo_adapted_to, test)
 
     def test_provides_bad(self):
         with self.assertRaises(Exception):
@@ -203,6 +242,73 @@ class InterfacesTest(unittest.TestCase):
             class Test(HasTraits):
                 pass
 
+    def test_provides_with_no_interface_check(self):
+
+        class Test(HasTraits):
+            # Deliberately _not_ implementing get_foo. This class
+            # should not pass an IFoo interface check.
+            pass
+
+        provides_ifoo = provides(IFoo)
+        with self.set_check_interfaces(0):
+            # Simulate application of the decorator
+            Test = provides_ifoo(Test)
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+
+    def test_provides_with_interface_check_warn(self):
+
+        class Test(HasTraits):
+            # Deliberately _not_ implementing get_foo. This class
+            # should not pass an IFoo interface check.
+            pass
+
+        provides_ifoo = provides(IFoo)
+        with self.set_check_interfaces(1):
+            with self.assertWarns(DeprecationWarning) as warnings_cm:
+                with self.assertLogs("traits", logging.WARNING):
+                    # Simulate application of the decorator
+                    Test = provides_ifoo(Test)
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+
+        self.assertIn(
+            "the @provides decorator will not perform interface checks",
+            str(warnings_cm.warning),
+        )
+
+        # Check we used the appropriate stacklevel.
+        _, _, this_module = __name__.rpartition(".")
+        self.assertIn(this_module, warnings_cm.filename)
+
+    def test_provides_with_interface_check_error(self):
+
+        class Test(HasTraits):
+            # Deliberately _not_ implementing get_foo. This class
+            # should not pass an IFoo interface check.
+            pass
+
+        provides_ifoo = provides(IFoo)
+        with self.set_check_interfaces(2):
+            with self.assertWarns(DeprecationWarning) as warnings_cm:
+                with self.assertRaises(InterfaceError):
+                    # Simulate application of the decorator
+                    Test = provides_ifoo(Test)
+
+        test = Test()
+        self.assertIsInstance(test, IFoo)
+
+        self.assertIn(
+            "the @provides decorator will not perform interface checks",
+            str(warnings_cm.warning),
+        )
+
+        # Check we used the appropriate stacklevel.
+        _, _, this_module = __name__.rpartition(".")
+        self.assertIn(this_module, warnings_cm.filename)
+
     def test_instance_adapt_no(self):
         ta = TraitsHolder()
 
@@ -317,3 +423,66 @@ class InterfacesTest(unittest.TestCase):
         provider = UndeclaredAverageProvider()
         with self.assertRaises(TraitError):
             ta.a_no = provider
+
+    def test_clone_supports_defaults(self):
+        i_foo_trait = Supports(IFoo)
+        new_default = Sample()
+        i_foo_clone = i_foo_trait(new_default)
+        default_value_kind, default_value = i_foo_clone.default_value()
+
+        # this is sub-optimal but matches Instance trait, see #1630
+        self.assertEqual(default_value_kind, DefaultValue.constant)
+        self.assertIsInstance(default_value, SampleFooAdapter)
+
+        ctrait_default = i_foo_clone.as_ctrait().default_value_for(None, "foo")
+        self.assertIsInstance(ctrait_default, SampleFooAdapter)
+
+    def test_clone_adapts_to_default(self):
+        i_foo_trait = AdaptsTo(IFoo)
+        new_default = Sample()
+        i_foo_clone = i_foo_trait(new_default)
+        default_value_kind, default_value = i_foo_clone.default_value()
+
+        # this is sub-optimal but matches Instance trait, see #1630
+        self.assertEqual(default_value_kind, DefaultValue.constant)
+        self.assertIs(default_value, new_default)
+
+        ctrait_default = i_foo_clone.as_ctrait().default_value_for(None, "foo")
+        self.assertIs(ctrait_default, new_default)
+
+    def test_subclass_supports_default(self):
+        s1 = TraitsHolderSub()
+        s2 = TraitsHolderSub()
+
+        # this is sub-optimal but matches Instance trait, see #1630
+        self.assertIs(s1.foo_adapted_to, s2.foo_adapted_to)
+
+    def test_subclass_adapts_to_default(self):
+        s1 = TraitsHolderSub()
+        s2 = TraitsHolderSub()
+
+        # this is sub-optimal but matches Instance trait, see #1630
+        self.assertIs(s1.foo_adapts_to, s2.foo_adapts_to)
+
+    @contextlib.contextmanager
+    def set_check_interfaces(self, check_interfaces_value):
+        """
+        Context manager to temporarily set has_traits.CHECK_INTERFACES
+        to the given value.
+
+        Parameters
+        ----------
+        check_interfaces_value : int
+            One of 0 (don't check), 1 (check and log a warning on interface
+            mismatch) or 2 (check and raise on interface mismatch).
+
+        Returns
+        -------
+        context manager
+        """
+        old_check_interfaces = has_traits.CHECK_INTERFACES
+        has_traits.CHECK_INTERFACES = check_interfaces_value
+        try:
+            yield
+        finally:
+            has_traits.CHECK_INTERFACES = old_check_interfaces
