@@ -1,20 +1,14 @@
-# ------------------------------------------------------------------------------
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
+# All rights reserved.
 #
-#  Copyright (c) 2007, Enthought, Inc.
-#  All rights reserved.
+# This software is provided without warranty under the terms of the BSD
+# license included in LICENSE.txt and may be redistributed only under
+# the conditions described in the aforementioned license. The license
+# is also available online at http://www.enthought.com/licenses/BSD.txt
 #
-#  This software is provided without warranty under the terms of the BSD
-#  license included in /LICENSE.txt and may be redistributed only
-#  under the conditions described in the aforementioned license.  The license
-#  is also available online at http://www.enthought.com/licenses/BSD.txt
-#
-#  Thanks for using Enthought open source!
-#
-# ------------------------------------------------------------------------------
+# Thanks for using Enthought open source!
 
-from __future__ import absolute_import
-
-from traits.testing.unittest_tools import unittest
+import unittest
 
 from traits.api import HasTraits, Str, Undefined, ReadOnly, Float
 
@@ -39,13 +33,11 @@ class UndefinedTestCase(unittest.TestCase):
     def test_initial_value(self):
         b = Bar()
         self.assertEqual(b.name, Undefined)
-        return
 
     def test_name_change(self):
         b = Bar()
         b.name = "first"
         self.assertEqual(b.name, "first")
-        return
 
     def test_read_only_write_once(self):
         f = Foo()
@@ -61,8 +53,6 @@ class UndefinedTestCase(unittest.TestCase):
         self.assertEqual(f.name, "second")
         self.assertEqual(f.original_name, "first")
 
-        return
-
     def test_read_only_write_once_from_constructor(self):
         f = Foo(name="first")
 
@@ -73,8 +63,3 @@ class UndefinedTestCase(unittest.TestCase):
         f.name = "second"
         self.assertEqual(f.name, "second")
         self.assertEqual(f.original_name, "first")
-
-        return
-
-
-### EOF #######################################################################
